/*******************************************************************************
 * Copyright (c) 2007 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.chart.reportitem;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.birt.report.model.api.elements.DesignChoiceConstants;

/**
 * The class defines some methods for font.
 */
public class FontHelper
{

	// Defines default font families.

	private static final String FONT_FAMILY_COURIER_NEW = "Courier New"; //$NON-NLS-1$

	private static final String FONT_FAMILY_IMPACT = "Impact"; //$NON-NLS-1$

	private static final String FONT_FAMILY_COMIC_SANS_MS = "Comic Sans MS"; //$NON-NLS-1$

	private static final String FONT_FAMILY_ARIAL = "Arial"; //$NON-NLS-1$

	private static final String FONT_FAMILY_TIMES_NEW_ROMAN = "Times New Roman"; //$NON-NLS-1$

	/**
	 * Map between CSS style font family to system font family
	 */
	public static Map familyMap = new HashMap( );

	/**
	 * Static table stores the font families. It provides the font name and the
	 * family of the fonts.
	 */
	static
	{
		familyMap.put( DesignChoiceConstants.FONT_FAMILY_SERIF,
				FONT_FAMILY_TIMES_NEW_ROMAN );
		familyMap.put( DesignChoiceConstants.FONT_FAMILY_SANS_SERIF,
				FONT_FAMILY_ARIAL );
		familyMap.put( DesignChoiceConstants.FONT_FAMILY_CURSIVE,
				FONT_FAMILY_COMIC_SANS_MS );
		familyMap.put( DesignChoiceConstants.FONT_FAMILY_FANTASY,
				FONT_FAMILY_IMPACT );
		familyMap.put( DesignChoiceConstants.FONT_FAMILY_MONOSPACE,
				FONT_FAMILY_COURIER_NEW );
	};

	/**
	 * Returns the final font family name.
	 * 
	 * @param fontFamily
	 *            specified font family name.
	 * @return actually font family name.
	 */
	public static String getFontFamily( String fontFamily )
	{
		String destFontName = (String) familyMap.get( fontFamily );

		if ( destFontName == null )
		{
			destFontName = fontFamily;
		}

		return destFontName;
	}
}
