/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.chart.ui.swt.wizard.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.birt.chart.model.data.Query;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Text;

/**
 * The manager for synchronizing data query, text and background color.
 */

public class DataDefinitionTextManager
{

	private static DataDefinitionTextManager instance;
	private HashMap textCollection = null;

	private DataDefinitionTextManager( )
	{
		textCollection = new HashMap( 10 );
	}

	public synchronized static DataDefinitionTextManager getInstance( )
	{
		if ( instance == null )
			instance = new DataDefinitionTextManager( );
		return instance;
	}

	public void addDataDefinitionText( Text text, Query query )
	{
		textCollection.put( text, query );
	}

	public void removeDataDefinitionText( Text text )
	{
		textCollection.remove( text );
	}

	public void removeAll( )
	{
		textCollection.clear( );
	}

	public void refreshAll( )
	{
		checkAll( );
		for ( Iterator iterator = textCollection.keySet( ).iterator( ); iterator.hasNext( ); )
		{
			Text text = (Text) iterator.next( );
			updateText( text );
		}
	}

	/**
	 * Checks all texts and removes disposed controls
	 * 
	 */
	private void checkAll( )
	{
		ArrayList listToRemove = new ArrayList( textCollection.size( ) );
		for ( Iterator iterator = textCollection.keySet( ).iterator( ); iterator.hasNext( ); )
		{
			Text text = (Text) iterator.next( );
			if ( text.isDisposed( ) )
			{
				listToRemove.add( text );
			}
		}
		for ( int i = 0; i < listToRemove.size( ); i++ )
		{
			textCollection.remove( listToRemove.get( i ) );
		}
	}

	public int getNumberOfSameDataDefinition( String expression )
	{
		checkAll( );
		int number = 0;
		for ( Iterator iterator = textCollection.keySet( ).iterator( ); iterator.hasNext( ); )
		{
			Text text = (Text) iterator.next( );
			if ( text.getText( ).equals( expression ) )
			{
				number++;
			}
		}
		return number;
	}

	public Text findText( Query query )
	{
		if ( textCollection.containsValue( query ) )
		{
			Iterator iterator = textCollection.entrySet( ).iterator( );
			while ( iterator.hasNext( ) )
			{
				Map.Entry entry = (Map.Entry) iterator.next( );
				if ( entry.getValue( ) == query )
				{
					return (Text) entry.getKey( );
				}
			}
		}
		return null;
	}

	public void updateText( Text text )
	{
		if ( textCollection.containsKey( text ) )
		{
			Query query = (Query) textCollection.get( text );
			text.setText( query.getDefinition( ) );
			Color color = ColorPalette.getInstance( )
					.getColor( text.getText( ) );
			text.setBackground( color );
		}
	}

	public void updateText( Query query )
	{
		Text text = findText( query );
		if ( text != null )
		{
			text.setText( query.getDefinition( ) );
			ColorPalette.getInstance( ).putColor( query.getDefinition( ) );
			text.setBackground( ColorPalette.getInstance( )
					.getColor( text.getText( ) ) );
		}
	}

	public void updateQuery( Text text )
	{
		if ( textCollection.containsKey( text ) )
		{
			Query query = (Query) textCollection.get( text );
			query.setDefinition( text.getText( ) );

			// Bind color to this data definition
			ColorPalette.getInstance( ).putColor( text.getText( ) );
			text.setBackground( ColorPalette.getInstance( )
					.getColor( text.getText( ) ) );
		}
	}
}
