
/*******************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/
package org.eclipse.birt.data.engine.olap.data.impl.aggregation.sort;

import java.util.logging.Logger;

import org.eclipse.birt.data.engine.olap.data.api.DimLevel;
import org.eclipse.birt.data.engine.olap.data.api.IAggregationResultSet;

/**
 * 
 */
public class SortKey
{

	private int[] aggrKeys;
	private int levelKeyIndex;
	private boolean[] aggrKeyAsc;
	private boolean levelKeyAsc;
	private boolean[] overAllDirection;
	private IAggregationResultSet targetResultSet;
	private int leveKeyOffset;
	private AxisQualifier[] qualifier;
	private AggregationResultRowNavigator[] filteredResultSet;
	private static Logger logger = Logger.getLogger( SortKey.class.getName( ) );
	
	//
	public SortKey( int[] aggrKeys, boolean[] aggrKeyAsc, int targetLevelKeyIndex, int targetLevelKeyOffset,
			IAggregationResultSet target, AxisQualifier[] qualifier )
	{
		Object[] params = {
				aggrKeys,
				aggrKeyAsc,
				new Integer( targetLevelKeyIndex ),
				new Integer( targetLevelKeyOffset ),
				target,
				qualifier
		};
		logger.entering( SortKey.class.getName( ), "SortKey", params );
		this.aggrKeys = aggrKeys;
		this.levelKeyIndex = targetLevelKeyIndex;
		this.leveKeyOffset = targetLevelKeyOffset;
		this.aggrKeyAsc = aggrKeyAsc;
		this.levelKeyAsc = target.getSortType( targetLevelKeyIndex ) == 1?false:true;
		this.qualifier = qualifier;
		this.targetResultSet = target;
		this.overAllDirection = new boolean[this.aggrKeyAsc.length+1];
		for( int i = 0; i < this.aggrKeyAsc.length; i++ )
		{
			this.overAllDirection[i] = this.aggrKeyAsc[i];
		}
		this.overAllDirection[this.overAllDirection.length-1] = this.levelKeyAsc;
		logger.exiting( SortKey.class.getName( ), "SortKey" );
	}
	
	/**
	 * 
	 * @return
	 */
	public DimLevel getLevel( )
	{
		return this.targetResultSet.getLevel( this.levelKeyIndex );
	}
	
	/**
	 * 
	 * @return
	 */
	public IAggregationResultSet getTargetResultSet( )
	{
		return this.targetResultSet;
	}
	
	/**
	 * 
	 * @return
	 */
	public boolean[] getOverallDirection()
	{
		return this.overAllDirection;
	}
	
	/**
	 * 
	 * @return
	 */
	public int[] getAggrKeys( )
	{
		return this.aggrKeys;
	}
	
	/**
	 * 
	 * @return
	 */
	public AxisQualifier[] getAxisQualifiers( )
	{
		return this.qualifier;
	}
	
	/**
	 * 
	 * @return
	 */
	public AggregationResultRowNavigator[] getFilteredResultSet( )
	{
		return this.filteredResultSet;
	}

	/**
	 * 
	 * @param resultSets
	 */
	public void setFilteredResultSet( AggregationResultRowNavigator[] resultSets )
	{
		this.filteredResultSet = resultSets;
	}
	
	/**
	 * 
	 * @return
	 */
	public int getLevelKeyIndex( )
	{
		return this.levelKeyIndex;
	}

	/**
	 * 
	 * @return
	 */
	public int getLevelKeyOffset( )
	{
		return this.leveKeyOffset;
	}
	
	/**
	 * 
	 * @return
	 */
	public boolean[] getAggrSortDirection( )
	{
		return this.aggrKeyAsc;
	}

	/**
	 * 
	 * @return
	 */
	public boolean getLevelSortDirection( )
	{
		return this.levelKeyAsc;
	}
}
